/*
 * ComparatorFactory.java
 * Copyright (c) 1999-2000 Andre Kaplan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


import java.util.Hashtable;
import org.gjt.sp.jedit.MiscUtilities;


/**
 * A comparator factory
 * @author Andre Kaplan
 * @version $Id: ComparatorFactory.java,v 1.4 2001/01/18 18:04:33 akaplan Exp $
**/
public class ComparatorFactory
{
    private static MiscUtilities.Compare defaultComparator =
        new UMLTreeDefaultComparator();
    private static MiscUtilities.Compare lineComparator =
         new UMLTreeLineComparator();


    public static MiscUtilities.Compare createUMLTreeDefaultComparator() {
        return defaultComparator;
    }


    public static MiscUtilities.Compare createUMLTreeLineComparator() {
        return lineComparator;
    }


    /**
     * Compares line numbers of two UMLTree nodes
    **/
    private static class UMLTreeLineComparator
        implements MiscUtilities.Compare
    {
        public int compare(Object o1,
                           Object o2)
        {
            int l1 = UMLTree.Node.getElement(o1).getLine();
            int l2 = UMLTree.Node.getElement(o2).getLine();

            if (l1 == l2) {
                return 0;
            } else if (l1 > l2) {
                return 1;
            } else {
                return -1;
            }
        }
    }


    /**
     * Compares names of two UMLTree nodes lexicographically
    **/
    private static class UMLTreeNameComparator
        implements MiscUtilities.Compare
    {
        public int compare(Object o1,
                           Object o2)
        {
            String s1 = UMLTree.Node.getElement(o1).getName();
            String s2 = UMLTree.Node.getElement(o2).getName();

            return s1.compareTo(s2);
        }
    }


    /**
     * Compares types of two UMLTree nodes.
     * Here is the default type hierarchy (from most to less important):
     * <OL>
     *   <LI>EXTENDS;
     *   <LI>IMPLEMENTS:
     *   <LI>METHOD;
     *   <LI>INTERFACE;
     *   <LI>CLASS;
     *   <LI>INNER_CLASS;
     *   <LI>ATTRIBUTE;
     *   <LI>THROWS;
     *   <LI>ERROR.
     * </OL>
    **/
    private static class UMLTreeTypeComparator
        implements MiscUtilities.Compare
    {
        public int compare(Object o1,
                           Object o2)
        {
            Object io1 = nodeTypes.get((Object)UMLTree.Node.getElement(o1).getElementType());
            Object io2 = nodeTypes.get((Object)UMLTree.Node.getElement(o2).getElementType());

            int i1 = (io1 == null) ? nodeTypes.size() : ((Integer)io1).intValue();
            int i2 = (io2 == null) ? nodeTypes.size() : ((Integer)io2).intValue();

            if (i1 == i2) {
                return 0;
            } else if (i1 > i2) {
                return 1;
            } else {
                return -1;
            }
        }


        static private Hashtable nodeTypes = new Hashtable();

        static {
            int order = 0;
            nodeTypes.put(UML.Type.EXTENDS,     new Integer(order++));
            nodeTypes.put(UML.Type.IMPLEMENTS,  new Integer(order++));
            nodeTypes.put(UML.Type.METHOD,      new Integer(order++));
            nodeTypes.put(UML.Type.INTERFACE,   new Integer(order++));
            nodeTypes.put(UML.Type.CLASS,       new Integer(order++));
            nodeTypes.put(UML.Type.INNER_CLASS, new Integer(order++));
            nodeTypes.put(UML.Type.ATTRIBUTE,   new Integer(order++));
            nodeTypes.put(UML.Type.THROWS,      new Integer(order++));
            nodeTypes.put(UML.Type.ERROR,       new Integer(order++));
        }
    }


    /**
     * Compares visibilities of two UMLTree nodes:
     * <OL>
     *   <LI>Public;
     *   <LI>Package;
     *   <LI>Protected;
     *   <LI>Private.
     * </OL>
    **/
    private static class UMLTreeVisibilityComparator
        implements MiscUtilities.Compare
    {
        public int compare(Object o1,
                           Object o2)
        {
            int m1 = UMLTree.Node.getElement(o1).getMod();
            int m2 = UMLTree.Node.getElement(o2).getMod();

            int mo1 = nodeMods[RWModifier.getVisLevelIndex(m1)];
            int mo2 = nodeMods[RWModifier.getVisLevelIndex(m2)];

            if (mo1 == mo2) {
                return 0;
            } else if (mo1 > mo2) {
                return 1;
            } else {
                return -1;
            }
        }


        static private int[] nodeMods = null;

        static {
            nodeMods = new int[4];
            int order = 0;
            nodeMods[RWModifier.MEMBER_VIS_PUBLIC]    = order++;
            nodeMods[RWModifier.MEMBER_VIS_PACKAGE]   = order++;
            nodeMods[RWModifier.MEMBER_VIS_PROTECTED] = order++;
            nodeMods[RWModifier.MEMBER_VIS_PRIVATE]   = order++;
        }
    }


    /**
     * Default comparator to sort UMLTree nodes. Multi-criteria sort order:
     * <OL>
     * 	<LI>By type (interface, classes, extensions, implementations, methods,
     *      inner or nested class, attributes, throws)
     *  <LI>By lexicographic order
     *  <LI>By access (public, package, protected, private)
     * </OL>
    **/
    private static class UMLTreeDefaultComparator
        implements MiscUtilities.Compare
    {
        public int compare(Object o1,
                           Object o2)
        {
            return this.cmp.compare(o1, o2);
        }


        private MiscUtilities.Compare cmp =
            new CompoundComparator(
                new MiscUtilities.Compare[] {
                     new UMLTreeTypeComparator()
                    ,new UMLTreeNameComparator()
                    ,new UMLTreeVisibilityComparator()
                }
            );
    }


    /**
     * Comparator to build multi-criteria comparators
    **/
    private static class CompoundComparator
        implements MiscUtilities.Compare
    {
        private MiscUtilities.Compare[] cmps = null;


        private CompoundComparator() {}


        public CompoundComparator(MiscUtilities.Compare[] comparators) {
            this.cmps = comparators;
        }


        public int compare(Object o1,
                           Object o2)
        {
            int cmp = 0;
            for (int i = 0; i < this.cmps.length; i++) {
                cmp = cmps[i].compare(o1, o2);
                if (cmp != 0) { return cmp; }
            }
            return cmp;
        }
    }
}


/*
 * ChangeLog:
 * $Log: ComparatorFactory.java,v $
 * Revision 1.4  2001/01/18 18:04:33  akaplan
 * Updated for jEdit 3.0, converted to UNIX line endings, some bugs corrected, JBrowse.Activator and JEditActivator removed
 *
 * Revision 1.2  2001/01/08 15:26:33  andre
 * Code cleanings
 *
 * Revision 1.1.1.1  2000/11/12 14:31:21  andre
 * JBrowse 1.3.1 initial import
 *
 * Revision 1.2  2000/09/02 22:51:46  akaplan
 * JBrowse 1.2: added sort option, automatic reparse feature, important internal changes, see Change_History.html for details
 *
*/
