/*
 * JBrowse.java - JBrowse GUI and Engine
 *
 * Copyright (c) 1999-2000 George Latkiewicz, Andre Kaplan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.text.Element;
import javax.swing.text.Position;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;

import org.gjt.sp.jedit.EBComponent;
import org.gjt.sp.jedit.EBMessage;
import org.gjt.sp.jedit.EditBus;

import org.gjt.sp.jedit.Buffer;
import org.gjt.sp.jedit.EditPane;
import org.gjt.sp.jedit.GUIUtilities;
import org.gjt.sp.jedit.View;

import org.gjt.sp.jedit.msg.BufferUpdate;
import org.gjt.sp.jedit.msg.EditPaneUpdate;

import org.gjt.sp.util.Log;


/**
 * The class that defines the main gui panel for the JBrowse plugin.
 * @author George Latkiewicz
 * @author Andre Kaplan
 * @version $Id: JBrowse.java,v 1.8 2001/01/18 18:04:33 akaplan Exp $
**/
public class JBrowse extends JPanel implements EBComponent
{
    /* public instance attributes */
    public JBrowseParser.Results results;

    /* private instance attributes */
    private View view;
    private PropertyAccessor props;
    private JBrowseParser.LineSource ls;
    private JBrowseParser parser;
    private UMLTree umlTree;

    private UMLTree.Model treeModel; // set on treeInit()
    private UMLTree.Node root;

    // status and action bar
    private JBrowseStatusPane statusPanel;
    private JBrowseActionPane actionPanel;

    private JBrowseOptionDialog optionDialog;
    private Options options;
    private Options.Filter  filterOpt;
    private Options.Display displayOpt;

    private ActionListener resizeAction      = null;
    private ActionListener parseAction       = null;
    private ActionListener showOptionsAction = null;
    private ActionListener showHelpAction    = null;

    private ActionListener statusBarOptionAction  = null;
    private ActionListener sortOptionAction       = null;
    private ActionListener filterOptionAction     = null;
    private ActionListener displayOptionAction    = null;

    private UMLTreeHandler        umlTreeHandler          = null;
    private TreeExpansionListener umlTreeExpansionHandler = null;
    private FocusListener         textAreaFocusHandler    = null;

    private JScrollPane scpTreePane;

    private Runnable parseRunnable = new Runnable() {
        public void run() {
            JBrowse.this.results = JBrowse.this.parser.parse();
            JBrowse.this.showResults(JBrowse.this.results);
        }
    };


    /**
     * Main JBrowse constructor
    **/
    public JBrowse(View view, ActionListener resize) {
        super();
        this.init(view, resize);
    }


    /**
     * This method creates a new instance of JBrowse GUI and Parsing engine.
    **/
    public void init(View view, ActionListener resize) {
        this.view = view;
        this.props = new JEditPropertyAccessor();
        this.ls = new JEditLineSource(view);
        if (view.getBuffer().getName().toLowerCase().endsWith(".java")) {
            this.parser = new JBrowseLineParser(this.ls);
        } else {
            this.parser = new JBrowseNullParser(
                this.ls.getName(), this.ls.getPath()
            );
        }
        this.umlTree = new UMLTree();
        this.umlTreeHandler = new UMLTreeHandler();

        this.resizeAction = resize;

        this.actionPanel = new JBrowseActionPane(this);
        this.statusPanel = new JBrowseStatusPane();

        this.setLayout(new BorderLayout());

        // Connect to the optionPane's options
        this.optionDialog =
            new JBrowseOptionDialog(this, "JBrowse - Configure Options");
        this.options = this.optionDialog.getOptions();
        this.filterOpt  = this.options.getFilterOptions();
        this.displayOpt = this.options.getDisplayOptions();

        String fileName = this.parser.getSourceName();

        // set root node to file name
        if (!fileName.toLowerCase().endsWith(".java") ) {
            fileName += " (NON-java file)";
            this.actionPanel.setErrorText("Un-parsed");
            this.actionPanel.validate();
        }

        this.treeInit(fileName);

        // Configure Parser
        this.parser.setOptions(this.options);
        this.parser.setRootNode(this.root);

        this.parse();
    }


    public void addNotify() {
        super.addNotify();
        EditBus.addToBus(this);
        this.addHandlers();
    }


    public void removeNotify() {
        super.removeNotify();
        EditBus.removeFromBus(this);
        this.removeHandlers();
    }


    public void dispose() {
        this.optionDialog.dispose();
    }


    public Options.DisplayIro getDisplayOptions() {
        return this.displayOpt;
    }


    public PropertyAccessor getPropertyAccessor() {
        return this.props;
    }


    private void parse() {
        this.parse(this.view, this.view.getBuffer());
    }


    private void parse(View view, Buffer buffer) {
        if (buffer.getName().toLowerCase().endsWith(".java")) {
            if (buffer.isLoaded()) {
                SwingUtilities.invokeLater(JBrowse.this.parseRunnable);
            } else {
                this.results = new JBrowseParser.Results();
                this.showResults(this.results);
                EditBus.addToBus(new BufferParseAction(view, buffer));
            }
        } else {
            this.results = new JBrowseParser.Results();
            this.showResults(this.results);
        }
    }


    /**
     * Reparse the current buffer
    **/
    private void reparse() {
        this.reparse(this.view, this.view.getBuffer());
    }


    /**
     * Reparse the given buffer in the given view
    **/
    private void reparse(View view, Buffer buffer) {
        if (buffer.getName().toLowerCase().endsWith(".java")) {
            if (this.parser instanceof JBrowseNullParser) {
                this.parser = new JBrowseLineParser(this.ls);

                this.treeInit(this.parser.getSourceName());
                this.parser.setOptions(options);
                this.parser.setRootNode(root);
            }

            if (buffer.isLoaded()) {
                SwingUtilities.invokeLater(JBrowse.this.parseRunnable);
            } else {
                this.results = new JBrowseParser.Results();
                this.showResults(this.results);
                EditBus.addToBus(new BufferParseAction(view, buffer));
            }
        }
    }


    /**
     * Reparse the current buffer if the automatic reparse option is enabled
    **/
    public void automaticReparse() {
        this.automaticReparse(this.view, this.view.getBuffer());
    }


    /**
     * Reparse the given buffer in the given view
     * if the automatic reparse option is enabled
    **/
    private void automaticReparse(View view, Buffer buffer) {
        boolean automaticParse = this.options.getAutomaticParse();
        if (automaticParse && this.isShowing()) {
            this.reparse(view, buffer);
        }
    }


    /**
     * Shows the option dialog
    **/
    private void showOptions() {
        this.optionDialog.init();

        // Determine size and position of the OptionDialog
        Dimension screen = getToolkit().getScreenSize();
        Dimension optSize = this.optionDialog.getSize();
        // JDK 1.2: replace getLocation().x with getX()
        int treeX = this.getLocation().x;
        int optY  = (screen.height - optSize.height) / 2;

        if (treeX + this.getSize().width + 12 + optSize.width > screen.width ) {
            // i.e. won't fit at right so...
            if (treeX - optSize.width - 12 < 0 ) {
                // i.e. won't fit at left either so overlap at right end
                this.optionDialog.setLocation( screen.width - optSize.width - 2, optY);
            } else {
                // left of
                this.optionDialog.setLocation( treeX - optSize.width - 10, optY);
            }
        } else {
            // right of
            this.optionDialog.setLocation( treeX + getSize().width + 10, optY);
        }

        this.optionDialog.setVisible(true);
        this.optionDialog.paintAll(this.optionDialog.getGraphics());

        // Repaint the JBrowse Dialog, required to eliminate
        // any contamination from the OptionDialog (really!)
        this.paintAll(this.getGraphics());
    }


    /**
     * Shows JBrowse help in the jEdit help viewer
    **/
    private void showHelp() {
        java.net.URL helpUrl = JBrowse.class.getResource("index.html");
        if (helpUrl == null) {
            Log.log(Log.NOTICE, this,
                "Help URL is null, cannot display help");
        } else {
            org.gjt.sp.jedit.gui.HelpViewer.gotoURL(helpUrl);
        }
    }


    /**
     * EBComponent implementation
     * Handles
     * <UL>
     *   <LI>EditPaneUpdate.BUFFER_CHANGED: Calls for automatic reparse
     *   <LI>EditPaneUpdate.CREATED: Adds to the EditPane textArea a JBrowse
     *   focus listener to do automatic reparse on focus
     *   <LI>EditPaneUpdate.DESTROYED: Removes from the focus listener
     * </UL>
    **/
    public void handleMessage(EBMessage message) {
        if (message instanceof EditPaneUpdate) {
            EditPaneUpdate epu = (EditPaneUpdate)message;
            View v = ((EditPane) epu.getSource()).getView();

            Log.log(Log.DEBUG, this, "***** EditPaneUpdate " + v);
            if (v == this.view) {
                if (epu.getWhat() == EditPaneUpdate.CREATED)
                {
                    epu.getEditPane().getTextArea().addFocusListener(
                        this.getTextAreaFocusHandler()
                    );
                }
                else if (epu.getWhat() == EditPaneUpdate.DESTROYED)
                {
                    epu.getEditPane().getTextArea().removeFocusListener(
                        this.getTextAreaFocusHandler()
                    );
                }
                else if (epu.getWhat() == EditPaneUpdate.BUFFER_CHANGED)
                {
                    this.automaticReparse();
                }
            }
        }
    }


    public Dimension getPreferredSize() {
        actionPanel.validate();
        Dimension dPanel    = this.getSize();
        Dimension dNewPanel = new Dimension(0,0);
        Dimension dViewCur  = scpTreePane.getViewport().getSize();
        Dimension dViewPref = scpTreePane.getViewport().getPreferredSize();
        Dimension dTreePref = umlTree.getPreferredSize();

        dNewPanel.height += actionPanel.getPreferredSize().height;

        // Set vertical scrollbar to visible, if it should be
        if (dViewCur.height < dViewPref.height) {
            scpTreePane.getVerticalScrollBar().setVisible(true);
        }

        // Set width to preferred width based on view and scroll bar
        if ( scpTreePane.getVerticalScrollBar().isVisible() ) {
            // i.e. 15 = scpTreePane.getVerticalScrollBar().getSize().width;
            dNewPanel.width = dViewPref.width + 12 + 16;
        } else {
            dNewPanel.width = dViewPref.width + 12;
        }
        dNewPanel.height += dViewPref.height;

        // Adjust width for status panel, if visible
        if ( statusPanel.isVisible() ) {
            statusPanel.validate();
            dNewPanel.width = Math.max(dNewPanel.width, statusPanel.getPreferredSize().width + 8);
            dNewPanel.height += statusPanel.getPreferredSize().height;
        }

        // Adjust width for top panel
        dNewPanel.width = Math.max( dNewPanel.width, actionPanel.getPreferredSize().width + 8);
        dNewPanel.height = Math.max(dNewPanel.height, dPanel.height);
        return dNewPanel;
    }


    public void setPreferredSize() {
        setSize(this.getPreferredSize());
        paintAll(getGraphics());
    }


    public boolean isStatusVisible() {
        return this.statusPanel.isVisible();
    }


    public void setStatusVisible(boolean visible) {
        this.statusPanel.setVisible(visible);
    }


    /**
     * This method should be called after every parse. It updates the text of
     * the status bar's labels to the results counts, displays the error
     * indicator (if appropriate) and then calls umlTree.display() to reload
     * the tree.
     */
    public void showResults(JBrowseParser.Results results) {
        umlTree.display(treeModel, options, results);

        if (results != null) { // required until File parser is implemented?
            // Update Status Bar
            this.statusPanel.showResults(results);

            // Update Parse Error Indicator
            if (results.getErrorCount() > 0) {
                actionPanel.setErrorText(results.getErrorCount() + " error(s)");
                actionPanel.setErrorVisible(true);
                actionPanel.validate();

                Dimension dFrame    = getSize();
                Dimension dActionPanel = actionPanel.getPreferredSize();
                if (dFrame.width < dActionPanel.width + 8) {
                    setPreferredSize();
                }

            } else {
                actionPanel.setErrorVisible(false);
            }
        }
    }


    /**
     * The initialization process for the tree, called as the last
     * step of the constructor.
     */
    private void treeInit(String fileName)
    {
        this.removeAll();

        if (this.scpTreePane == null) {
            this.scpTreePane = new JScrollPane();
        }
        this.scpTreePane.setViewportView(this.umlTree);

        this.add(this.actionPanel, BorderLayout.NORTH);
        this.add(this.scpTreePane, BorderLayout.CENTER);
        this.add(this.statusPanel, BorderLayout.SOUTH);

        if (options.getShowStatusBar()) {
            this.statusPanel.setVisible(true);
        } else {
            this.statusPanel.setVisible(false);
        }

        // Setup Tree
        this.root = new UMLTree.Node(fileName);

        // get tree model and tree
        this.treeModel = new UMLTree.Model(this.root);
    }


    /**
     * Adds the listeners to provide the sort and automatic reparse features
    **/
    private void addHandlers() {
        this.umlTree.addTreeSelectionListener(this.umlTreeHandler);
        this.umlTree.addMouseListener(this.umlTreeHandler);

        this.umlTree.addTreeExpansionListener(
                this.getUMLTreeExpansionHandler()
        );

        EditPane[] editPanes = this.view.getEditPanes();
        for (int i = 0; i < editPanes.length; i++) {
            editPanes[i].getTextArea().addFocusListener(
                this.getTextAreaFocusHandler()
            );
        }
    }


    /**
     * Removes the listeners that provided the sort and automatic reparse
     * features
    **/
    private void removeHandlers() {
        this.umlTree.removeTreeSelectionListener(this.umlTreeHandler);
        this.umlTree.removeMouseListener(this.umlTreeHandler);

        this.umlTree.removeTreeExpansionListener(
            this.getUMLTreeExpansionHandler()
        );

        EditPane[] editPanes = this.view.getEditPanes();
        for (int i = 0; i < editPanes.length; i++) {
            editPanes[i].getTextArea().removeFocusListener(
                this.getTextAreaFocusHandler()
            );
        }
    }


    public ActionListener getResizeAction() {
        return this.resizeAction;
    }


    public ActionListener getParseAction() {
        if (this.parseAction == null) {
            this.parseAction = new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    JBrowse.this.reparse();
                }
            };
        }

        return this.parseAction;
    }


    public ActionListener getShowOptionsAction() {
        if (this.showOptionsAction == null) {
            this.showOptionsAction = new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    JBrowse.this.showOptions();
                }
            };
        }

        return this.showOptionsAction;
    }


    public ActionListener getShowHelpAction() {
        if (this.showHelpAction == null) {
            this.showHelpAction = new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    JBrowse.this.showHelp();
                }
            };
        }

        return this.showHelpAction;
    }


    public ActionListener getStatusBarOptionAction() {
        if (this.statusBarOptionAction == null) {
            this.statusBarOptionAction = new StatusBarOptionAction();
        }

        return this.statusBarOptionAction;
    }


    public ActionListener getSortOptionAction() {
        if (this.sortOptionAction == null) {
            this.sortOptionAction = new SortOptionAction();
        }

        return this.sortOptionAction;
    }


    public ActionListener getFilterOptionAction() {
        if (this.filterOptionAction == null) {
            this.filterOptionAction = new FilterOptionAction();
        }

        return this.filterOptionAction;
    }


    public ActionListener getDisplayOptionAction() {
        if (this.displayOptionAction == null) {
            this.displayOptionAction = new DisplayOptionAction();
        }

        return this.displayOptionAction;
    }


    private TreeExpansionListener getUMLTreeExpansionHandler() {
        if (this.umlTreeExpansionHandler == null) {
            this.umlTreeExpansionHandler = new UMLTreeExpansionHandler();
        }

        return this.umlTreeExpansionHandler;
    }


    private FocusListener getTextAreaFocusHandler() {
        if (this.textAreaFocusHandler == null) {
            this.textAreaFocusHandler = new TextAreaFocusHandler();
        }

        return this.textAreaFocusHandler;
    }


    private class StatusBarOptionAction implements ActionListener
    {
        public void actionPerformed(ActionEvent e) {
            JBrowse.this.setStatusVisible(
                JBrowse.this.options.getShowStatusBar()
            );
        }
    }


    private class SortOptionAction implements ActionListener
    {
        public void actionPerformed(ActionEvent e) {
            if (JBrowse.this.root.getChildCount() > 0) {
                JBrowse.this.umlTree.sortToggled(JBrowse.this.options);
            }
        }
    }


    private class FilterOptionAction implements ActionListener
    {
        public void actionPerformed(ActionEvent e) {
            if (JBrowse.this.root.getChildCount() > 0) {
                // there are nodes below the root, therefore need to reload
                JBrowse.this.umlTree.display(
                    JBrowse.this.treeModel,
                    JBrowse.this.options,
                    JBrowse.this.results
                );
            }
        }
    }


    private class DisplayOptionAction implements ActionListener
    {
        public void actionPerformed(ActionEvent e) {
            if (JBrowse.this.root.getChildCount(JBrowse.this.filterOpt) > 0) {
                JBrowse.this.umlTree.updateVisibleToggled(options);
            }
        }
    }


    private class UMLTreeExpansionHandler implements TreeExpansionListener
    {
        public void treeCollapsed(TreeExpansionEvent evt) {
            Log.log(Log.DEBUG, this, "Tree Collapsed");
        }


        public void treeExpanded(TreeExpansionEvent evt) {
            Log.log(Log.DEBUG, this, "Tree Expanded");

            org.gjt.sp.jedit.MiscUtilities.Compare compare;
            if (JBrowse.this.options.getSort()) {
                compare = ComparatorFactory.createUMLTreeDefaultComparator();
            } else {
                compare = ComparatorFactory.createUMLTreeLineComparator();
            }

            TreePath path = evt.getPath();
            UMLTree.Node node = (UMLTree.Node) path.getLastPathComponent();
            UMLTreeSorter.sort(node, compare);

            DefaultTreeModel model = (DefaultTreeModel) JBrowse.this.umlTree.getModel();
            model.nodeStructureChanged(node);
        }
    }


    /**
     * This class handles communication between jEdit and JBrowse tree
     * for mouse events and tree slection events
    **/
    private class UMLTreeHandler implements MouseListener, TreeSelectionListener
    {
        public void mouseClicked(MouseEvent e) {
            int selRow = JBrowse.this.umlTree.getRowForLocation(e.getX(), e.getY());

            // If user clicked outside of the text, returns
            if (selRow == -1) {
                return;
            }

            // Only required to supplement valueChanged(TreeSelectionEvent evt)
            // when mouse clicked on same node.
            if (   (selRow == JBrowse.this.umlTree.getMaxSelectionRow())
                && (e.getClickCount() == 1)
            ) {
                // Check if view buffer matches tree buffer
                if (!this.verifyBuffer()) {
                    return;
                }

                UMLTree.Node selectedNode = (UMLTree.Node)
                    JBrowse.this.umlTree.getLastSelectedPathComponent();

                this.setPosition(selectedNode);
            }
        }


        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseReleased(MouseEvent e) {}


        public void valueChanged(TreeSelectionEvent evt) {
            UMLTree.Node selectedNode;

            if (JBrowse.this.umlTree.isSelectionEmpty() ) {
                return;
            }

            // Check if view buffer matches tree buffer
            if (!this.verifyBuffer()) {
                return;
            }

            selectedNode = (UMLTree.Node)
                JBrowse.this.umlTree.getSelectionPath().getLastPathComponent();

            this.setPosition(selectedNode);
        }


        /**
         * Changes the position of the cursor in the buffer to match the tree
         * node. The line containing the match is selected and appears near the top
         * of the textarea
        **/
        private void setPosition(UMLTree.Node node) {
            Position pos = (Position) node.getPosition();
            if (pos == null) {
                return;
            }

            Element map = ((Buffer) JBrowse.this.ls.getObject()).getDefaultRootElement();
            Element lineElement = map.getElement(map.getElementIndex(pos.getOffset()));
            if (lineElement == null) {
                return;
            }

            int origin = Math.max(0, map.getElementIndex(pos.getOffset()) - 4);
            JBrowse.this.view.getTextArea().setFirstLine(origin);
            JBrowse.this.view.getTextArea().select(
                lineElement.getStartOffset(), lineElement.getEndOffset() - 1
            );
        }


        /**
         * Checks if the view buffer matches tree buffer
        **/
        private boolean verifyBuffer() {
            if (JBrowse.this.view.getBuffer() == JBrowse.this.ls.getObject()) {
                return true;
            }

            String bufferWant = JBrowse.this.ls.getPath();
            String bufferHave = JBrowse.this.view.getBuffer().getPath();
            GUIUtilities.error(
                JBrowse.this.view,
                "jbrowse.msg.wrongBuffer",
                new Object[] { bufferWant, bufferHave }
            );
            return false;
        }
    }


    /**
     * JBrowse acts on the last focused TextArea
    **/
    private class TextAreaFocusHandler extends FocusAdapter {
        public void focusGained(FocusEvent evt) {
            // walk up component hierarchy, looking for an EditPane
            Component comp = (Component) evt.getSource();
            while(!(comp instanceof EditPane)) {
                if (comp == null) {
                    Log.log(Log.ERROR, this, "Could not find EditPane");
                    return;
                }
                comp = comp.getParent();
            }

            EditPane editPane = (EditPane) comp;
            // No need to reparse if the buffer was already parsed
            if (editPane.getBuffer().getPath().equals(
                    JBrowse.this.parser.getSourcePath())
            ) {
                return;
            }
            if (editPane.getBuffer().isLoaded()) {
                JBrowse.this.automaticReparse(
                    editPane.getView(), editPane.getBuffer()
                );
            }
        }
    }


    private class BufferParseAction implements EBComponent
    {
        private View   view;
        private Buffer buffer;


        public BufferParseAction(View view, Buffer buffer) {
            this.view   = view;
            this.buffer = buffer;
        }


        public void handleMessage(EBMessage message) {
            if (message instanceof BufferUpdate) {
                BufferUpdate bu = (BufferUpdate) message;
                if (    (bu.getWhat() == BufferUpdate.LOADED)
                    &&  (this.buffer == bu.getBuffer())
                ) {
                    SwingUtilities.invokeLater(JBrowse.this.parseRunnable);
                    EditBus.removeFromBus(this);
                }
            }
        }
    }
}


/*
 * ChangeLog:
 * $Log: JBrowse.java,v $
 * Revision 1.8  2001/01/18 18:04:33  akaplan
 * Updated for jEdit 3.0, converted to UNIX line endings, some bugs corrected, JBrowse.Activator and JEditActivator removed
 *
 * Revision 1.12  2001/01/18 17:39:27  andre
 * Reworked parse action
 *
 * Revision 1.11  2001/01/18 14:49:19  andre
 * Removed .* imports
 *
 * Revision 1.10  2001/01/18 14:23:06  andre
 * JBrowse.Activator and JEditActivator removed
 *
 * Revision 1.9  2001/01/18 13:43:36  andre
 * Preparing JBrowse.Activator, JEditActivator removal
 *
 * Revision 1.8  2001/01/18 13:31:10  andre
 * UMLTree mouse/selection handlers management changed
 *
 * Revision 1.7  2001/01/17 16:54:03  andre
 * JBrowse properly added/removed from EditBus
 *
 * Revision 1.6  2001/01/08 15:26:33  andre
 * Code cleanings
 *
 * Revision 1.5  2001/01/08 14:32:26  andre
 * Converted line endings to UNIX
 *
 * Revision 1.4  2001/01/08 13:53:34  andre
 * sort option should be fixed
 *
 * Revision 1.3  2000/11/19 15:03:18  andre
 * Doc updates, props updates and  small code cleanings for JBrowse 1.3.2
 *
 * Revision 1.2  2000/11/19 12:49:17  andre
 * Fixed sort bug with JDK1.3 and Windows; changing the sort order doesn't collapse the JBrowse tree anymore
 *
 * Revision 1.1.1.1  2000/11/12 14:31:21  andre
 * JBrowse 1.3.1 initial import
 *
 * Revision 1.6  2000/10/27 00:50:40  akaplan
 * Fixed bug #115158 and #117727
 *
 * Revision 1.5  2000/09/11 00:30:44  akaplan
 * Automatic reparse was broken when a new buffer was loaded
 *
 * Revision 1.4  2000/09/02 22:51:46  akaplan
 * JBrowse 1.2: added sort option, automatic reparse feature, important internal changes, see Change_History.html for details
 *
*/
