/*
 * JBrowseOptionPane.java - JBrowse options panel
 *
 * Copyright (c) 1999-2000 George Latkiewicz, Andre Kaplan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.border.Border;
import javax.swing.border.BevelBorder;
import javax.swing.border.TitledBorder;

import org.gjt.sp.jedit.AbstractOptionPane;

import org.gjt.sp.util.Log;


/**
 * JBrowse option pane
 * @author George Latkiewicz
 * @author Andre Kaplan
 * @version $Id: JBrowseOptionPane.java,v 1.5 2001/01/18 18:04:33 akaplan Exp $
**/
public class JBrowseOptionPane extends AbstractOptionPane
{
    // protected members inherited from AbstractOptionPane: y, gridBag

    // private state
    boolean isInitGui;
    boolean isInitModel;

    // private gui components

    // general options
    private JCheckBox cbxStatusBar;
    private JCheckBox cbxAutomaticParse;
    private JCheckBox cbxSort;

    // filter options
    private JCheckBox cbxShowAttributes;
    private JCheckBox cbxShowPrimitives;
    private JCheckBox cbxShowGeneralizations;
    private JCheckBox cbxShowThrows;

    private JComboBox cmbTopLevelVis;
    private JComboBox cmbMemberVis;
    private int topLevelVisIndex;
    private int memberVisIndex;

    // display options
    private JCheckBox cbxShowArguments;
    private JCheckBox cbxShowArgumentNames;
    private JCheckBox cbxShowNestedName;
    private JCheckBox cbxShowIconKeywords;
    private JCheckBox cbxShowMiscMod;
    private JCheckBox cbxShowLineNum;

    private JComboBox cmbStyle;
    private int styleIndex = Options.Display.STYLE_UML;

    private JCheckBox cbxVisSymbols;
    private JCheckBox cbxAbstractItalic;
    private JCheckBox cbxStaticUlined;
    private JCheckBox cbxTypeIsSuffixed;

    // Options object
    private Options options = new Options();
    private Options.Filter  filterOpt  = options.getFilterOptions();
    private Options.Display displayOpt = options.getDisplayOptions();

    // Property Accessor
    private PropertyAccessor props;

    private boolean batchUpdate = false;

    // Listeners
    private ActionListener defaultAction       = null;
    private ActionListener updateOptionsAction = null;
    private ActionListener setOptionsAction    = null;


    public JBrowseOptionPane() {
        this("jbrowse");
    }


    public JBrowseOptionPane(String title)
    {
        super(title);
        setLayout(gridBag = new GridBagLayout());

        // It is the instantiating code's responsibility to call:
        // initGui(), initModel(), and setOptions() before displaying
        // Also either addDefaultListeners or addJBrowseListeners must be
        // called to allow the GUI to correctly respond to user interaction
    } // public JBrowseOptionPane(String): <init>


    /**
     * AbstractOptionPane implementation (_init and _save)
    **/
    public void _init()
    {
        this.setPropertyAccessor(new JEditPropertyAccessor());
        options.load(props);

        initGui();   // setup display from property values
        initModel(); // set GUI to model (as defined in Options object)
    }


    /**
     * AbstractOptionPane implementation (_init and _save)
     * The method called by the File->Plugin Options save button for
     * setting the JBrowse plugin options for all future sessions.
     * It saves the current view state to the associated property values.
    **/
    public void _save() {
        options.save(props);
    }


    PropertyAccessor getPropertyAccessor() { return props; }


    void setPropertyAccessor(PropertyAccessor props) {
        this.props = props;
    }


    public boolean isInitGui() { return isInitGui; }


    public boolean isInitModel() { return isInitModel; }


    /**
     * Sets this OptionPane's options object to the state specified by the
     * the OptionPane's associated PropertyAccessor.
    **/
    public void load() {
        batchUpdate = true;
        options.load(props);
        batchUpdate = false;
    }


    /**
     * Setup the GUI (with no current state).
     * This should only be called once in the constructor for this
     * JBrowseOptionPane.
    **/
    private void initGui()
    {
        // -----
        // Title
        // -----
        JLabel titleLabel = new JLabel(
                props.getProperty("options." + getName() + ".panel_label") + ":",
                        JLabel.LEFT );
        titleLabel.setFont(new Font("Helvetica", Font.BOLD + Font.ITALIC, 13));

        addComponent(titleLabel);


        // ---------------
        // General Options
        // ---------------
        JPanel generalPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 9, 0));
        cbxStatusBar = new JCheckBox(
                props.getProperty("options.jbrowse.showStatusBar"));
        generalPanel.add(cbxStatusBar);

        cbxAutomaticParse = new JCheckBox(
                props.getProperty("options.jbrowse.automaticParse"));
        generalPanel.add(cbxAutomaticParse);

        cbxSort = new JCheckBox(
                props.getProperty("options.jbrowse.sort"));
        generalPanel.add(cbxSort);

        addComponent(generalPanel);

        // --------------
        // Filter Options
        // --------------
        OptionPanel filterPanel = new OptionPanel();
        filterPanel.setBorder(this.createOptionBorder(
            " " + props.getProperty("options.jbrowse.filterOptions") + " "
        ));

        /* Attributes */
        JPanel attrPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
        cbxShowAttributes = new JCheckBox(
                props.getProperty("options.jbrowse.showAttr") + " ");
        attrPanel.add(cbxShowAttributes);

        /* Primitive Attributes */
        cbxShowPrimitives = new JCheckBox(
                props.getProperty("options.jbrowse.showPrimAttr"));
        attrPanel.add(cbxShowPrimitives);
        filterPanel.addComponent(attrPanel);

        /* Generalizations */
        cbxShowGeneralizations = new JCheckBox(
                props.getProperty("options.jbrowse.showGeneralizations") + " ");
        filterPanel.addComponent(cbxShowGeneralizations);

        /* Throws */
        cbxShowThrows = new JCheckBox(
                props.getProperty("options.jbrowse.showThrows") + " ");
        filterPanel.addComponent(cbxShowThrows);

        /* Visibility Level */
        JLabel visLevelLabel = new JLabel(
                props.getProperty("options.jbrowse.visLevelLabel") );
        filterPanel.addComponent(visLevelLabel);

        /* Top-Level Visibility Options */
        String[] topLevelVisNames = { "package", "public" };
        cmbTopLevelVis = new JComboBox(topLevelVisNames);
        filterPanel.addComponent(props.getProperty("options.jbrowse.topLevelVis"),
            cmbTopLevelVis);

        /* Member-Level Visibility Options */
        String[] memberVisNames = { "private", "package", "protected", "public" };
        cmbMemberVis = new JComboBox(memberVisNames);
        filterPanel.addComponent(props.getProperty("options.jbrowse.memberVis"),
            cmbMemberVis);

        addComponent(filterPanel);


        // ---------------
        // Display Options
        // ---------------
        OptionPanel displayPanel = new OptionPanel();
        displayPanel.setBorder(this.createOptionBorder(
            " " + props.getProperty("options.jbrowse.displayOptions") + " "
        ));

        /* Arguments */
        JPanel argPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));

        cbxShowArguments = new JCheckBox(
                props.getProperty("options.jbrowse.showArgs") + " ");
        argPanel.add(cbxShowArguments);

        /* Argument Names */
        cbxShowArgumentNames = new JCheckBox(
                props.getProperty("options.jbrowse.showArgNames"));
        argPanel.add(cbxShowArgumentNames);
        displayPanel.addComponent(argPanel);

        /* qualify nested class/interface names */
        cbxShowNestedName = new JCheckBox(
                props.getProperty("options.jbrowse.showNestedName"));
        displayPanel.addComponent(cbxShowNestedName);

        /* class/interface modifiers */
        cbxShowIconKeywords = new JCheckBox(
                props.getProperty("options.jbrowse.showIconKeywords"));
        displayPanel.addComponent(cbxShowIconKeywords);

        /* misc. detail modifiers */
        cbxShowMiscMod = new JCheckBox(
                props.getProperty("options.jbrowse.showMiscMod"));
        displayPanel.addComponent(cbxShowMiscMod);

        /* Line Numbers */
        cbxShowLineNum = new JCheckBox(
                props.getProperty("options.jbrowse.showLineNums"));
        displayPanel.addComponent(cbxShowLineNum);

        /* Display Style */
        String[] styleNames = {
            props.getProperty("options.jbrowse.umlStyle"),
            props.getProperty("options.jbrowse.javaStyle"),
            props.getProperty("options.jbrowse.customStyle") };
        cmbStyle = new JComboBox(styleNames);
        displayPanel.addComponent(props.getProperty("options.jbrowse.displayStyle"),
            cmbStyle);

        /* Custom Display Style Options */
        JLabel customOptionsLabel = new JLabel(
                props.getProperty("options.jbrowse.customOptions"));
        displayPanel.addComponent(customOptionsLabel);

        cbxVisSymbols = new JCheckBox(
                props.getProperty("options.jbrowse.custVisAsSymbol"));

        cbxAbstractItalic = new JCheckBox(
                props.getProperty("options.jbrowse.custAbsAsItalic"));

        cbxStaticUlined = new JCheckBox(
                props.getProperty("options.jbrowse.custStaAsUlined"));

        cbxTypeIsSuffixed = new JCheckBox(
                props.getProperty("options.jbrowse.custTypeIsSuffixed"));


        displayPanel.addComponent(cbxVisSymbols);
        displayPanel.addComponent(cbxAbstractItalic);
        displayPanel.addComponent(cbxStaticUlined);
        displayPanel.addComponent(cbxTypeIsSuffixed);

        addComponent(displayPanel);

        this.addDefaultListeners();

        isInitGui = true;

    } // initGui(): void


    /**
     * This method sets the GUI representation of the model to the state
     * specified  by the current option object's state.
    **/
    public void initModel()
    {
        Log.log(Log.DEBUG, this, "initModel: " + this.getName());
        batchUpdate = true;

        // General Options
        cbxStatusBar.getModel().setSelected( options.getShowStatusBar() );
        cbxAutomaticParse.getModel().setSelected( options.getAutomaticParse() );
        cbxSort.getModel().setSelected( options.getSort() );

        // Filter Options
        cbxShowAttributes.getModel().setSelected( filterOpt.getShowAttributes() );
        cbxShowPrimitives.getModel().setSelected( filterOpt.getShowPrimitives() );
        cbxShowGeneralizations.getModel().setSelected( filterOpt.getShowGeneralizations() );
        cbxShowThrows.getModel().setSelected( filterOpt.getShowThrows() );

        cmbTopLevelVis.setSelectedIndex( filterOpt.getTopLevelVisIndex() );
        cmbMemberVis.setSelectedIndex(   filterOpt.getMemberVisIndex() );

        // Display Options
        cbxShowArguments.getModel().setSelected(     displayOpt.getShowArguments() );
        cbxShowArgumentNames.getModel().setSelected( displayOpt.getShowArgumentNames() );
        cbxShowNestedName.getModel().setSelected(    displayOpt.getShowNestedName() );
        cbxShowIconKeywords.getModel().setSelected(  displayOpt.getShowIconKeywords() );
        cbxShowMiscMod.getModel().setSelected(       displayOpt.getShowMiscMod() );
        cbxShowLineNum.getModel().setSelected(       displayOpt.getShowLineNum() );

        cmbStyle.setSelectedIndex(displayOpt.getStyleIndex() );

        cbxVisSymbols.getModel().setSelected(     displayOpt.getVisSymbols() );
        cbxAbstractItalic.getModel().setSelected( displayOpt.getAbstractItalic() );
        cbxStaticUlined.getModel().setSelected(   displayOpt.getStaticUlined() );
        cbxTypeIsSuffixed.getModel().setSelected( displayOpt.getTypeIsSuffixed() );


        // Set enabled/disabled on showArgumentNames, showPrimitives checkboxes
        if(cbxShowArguments.getModel().isSelected()) {
            cbxShowArgumentNames.getModel().setEnabled(true);
        } else {
            cbxShowArgumentNames.getModel().setSelected(false);
            cbxShowArgumentNames.getModel().setEnabled(false);
        }

        if(cbxShowAttributes.getModel().isSelected()) {
            cbxShowPrimitives.getModel().setEnabled(true);
        } else {
            cbxShowPrimitives.getModel().setSelected(false);
            cbxShowPrimitives.getModel().setEnabled(false);
        }

        refreshDisplayOptions(styleIndex);

        isInitModel = true;
        batchUpdate = false;
    } // initModel(): void


    public void addDefaultListeners() {
        ActionListener defaultListener = this.getDefaultAction();

        // general options
        this.cbxStatusBar.addActionListener(defaultListener);
        this.cbxStatusBar.addActionListener(defaultListener);
        this.cbxAutomaticParse.addActionListener(defaultListener);
        this.cbxSort.addActionListener(defaultListener);

        // filter options
        this.cbxShowAttributes.addActionListener(defaultListener);
        this.cbxShowPrimitives.addActionListener(defaultListener);
        this.cbxShowGeneralizations.addActionListener(defaultListener);
        this.cbxShowThrows.addActionListener(defaultListener);
        this.cmbTopLevelVis.addActionListener(defaultListener);
        this.cmbMemberVis.addActionListener(defaultListener);

        // display options
        this.cbxShowArguments.addActionListener(defaultListener);
        this.cbxShowArgumentNames.addActionListener(defaultListener);
        this.cbxShowNestedName.addActionListener(defaultListener);
        this.cbxShowIconKeywords.addActionListener(defaultListener);
        this.cbxShowMiscMod.addActionListener(defaultListener);
        this.cbxShowLineNum.addActionListener(defaultListener);

        this.cmbStyle.addActionListener(defaultListener);

        this.cbxVisSymbols.addActionListener(defaultListener);
        this.cbxAbstractItalic.addActionListener(defaultListener);
        this.cbxStaticUlined.addActionListener(defaultListener);
        this.cbxTypeIsSuffixed.addActionListener(defaultListener);
    }


    public void removeDefaultListeners() {
        ActionListener defaultListener = this.getDefaultAction();

        // general options
        this.cbxStatusBar.removeActionListener(defaultListener);
        this.cbxStatusBar.removeActionListener(defaultListener);
        this.cbxAutomaticParse.removeActionListener(defaultListener);
        this.cbxSort.removeActionListener(defaultListener);

        // filter options
        this.cbxShowAttributes.removeActionListener(defaultListener);
        this.cbxShowPrimitives.removeActionListener(defaultListener);
        this.cbxShowGeneralizations.removeActionListener(defaultListener);
        this.cbxShowThrows.removeActionListener(defaultListener);
        this.cmbTopLevelVis.removeActionListener(defaultListener);
        this.cmbMemberVis.removeActionListener(defaultListener);

        // display options
        this.cbxShowArguments.removeActionListener(defaultListener);
        this.cbxShowArgumentNames.removeActionListener(defaultListener);
        this.cbxShowNestedName.removeActionListener(defaultListener);
        this.cbxShowIconKeywords.removeActionListener(defaultListener);
        this.cbxShowMiscMod.removeActionListener(defaultListener);
        this.cbxShowLineNum.removeActionListener(defaultListener);

        this.cmbStyle.removeActionListener(defaultListener);

        this.cbxVisSymbols.removeActionListener(defaultListener);
        this.cbxAbstractItalic.removeActionListener(defaultListener);
        this.cbxStaticUlined.removeActionListener(defaultListener);
        this.cbxTypeIsSuffixed.removeActionListener(defaultListener);
    }


    /**
     * Allows this option pane to reflect any user interaction directly to
     * JBrowse
    **/
    public void addJBrowseListeners(JBrowse jbrowse) {
        // general options
        ActionListener statusBarOptionAction =
            this.createAction(jbrowse.getStatusBarOptionAction());
        this.cbxStatusBar.addActionListener(statusBarOptionAction);

        ActionListener resizeAction =
            this.createAction(jbrowse.getResizeAction());
        this.cbxStatusBar.addActionListener(resizeAction);

        this.cbxAutomaticParse.addActionListener(this.getDefaultAction());

        ActionListener sortOptionAction =
            this.createAction(jbrowse.getSortOptionAction());
        this.cbxSort.addActionListener(sortOptionAction);

        // filter options
        ActionListener filterOptionAction =
            this.createAction(jbrowse.getFilterOptionAction());
        this.cbxShowAttributes.addActionListener(filterOptionAction);
        this.cbxShowPrimitives.addActionListener(filterOptionAction);
        this.cbxShowGeneralizations.addActionListener(filterOptionAction);
        this.cbxShowThrows.addActionListener(filterOptionAction);
        this.cmbTopLevelVis.addActionListener(filterOptionAction);
        this.cmbMemberVis.addActionListener(filterOptionAction);

        // display options
        ActionListener displayOptionAction =
            this.createAction(jbrowse.getDisplayOptionAction());
        this.cbxShowArguments.addActionListener(displayOptionAction);
        this.cbxShowArgumentNames.addActionListener(displayOptionAction);
        this.cbxShowNestedName.addActionListener(displayOptionAction);
        this.cbxShowIconKeywords.addActionListener(displayOptionAction);
        this.cbxShowMiscMod.addActionListener(displayOptionAction);
        this.cbxShowLineNum.addActionListener(displayOptionAction);

        this.cmbStyle.addActionListener(displayOptionAction);

        this.cbxVisSymbols.addActionListener(displayOptionAction);
        this.cbxAbstractItalic.addActionListener(displayOptionAction);
        this.cbxStaticUlined.addActionListener(displayOptionAction);
        this.cbxTypeIsSuffixed.addActionListener(displayOptionAction);
    }


    private ActionListener getDefaultAction() {
        if (this.defaultAction == null) {
            this.defaultAction = this.createDefaultAction();
        }
        return this.defaultAction;
    }


    private ActionListener getUpdateOptionsAction() {
        if (this.updateOptionsAction == null) {
            this.updateOptionsAction = new UpdateOptionsAction();
        }
        return this.updateOptionsAction;
    }


    private ActionListener getSetOptionsAction() {
        if (this.setOptionsAction == null) {
            this.setOptionsAction = new SetOptionsAction();
        }
        return this.setOptionsAction;
    }


    private ActionListener createDefaultAction() {
        return this.createAction(null);
    }


    private ActionListener createAction(ActionListener jbrowseAction) {
        Condition batchUpdateCondition = new BatchUpdateCondition();

        if (jbrowseAction == null) {
            return (
                new CompoundAction(
                    this.getUpdateOptionsAction(),
                    new ConditionalAction(
                        batchUpdateCondition,
                        null,
                        this.getSetOptionsAction()
                    )
                )
            );
        } else {
            return (
                new CompoundAction(
                    this.getUpdateOptionsAction(),
                    new ConditionalAction(
                        batchUpdateCondition,
                        null,
                        new CompoundAction(
                            this.getSetOptionsAction(), jbrowseAction
                        )
                    )
                )
            );
        }
    }


    /**
     * Set the enabled and selected/index state of all the display options
     * that are dependant on the cmbStyle control. The state to be set to
     * is determined by the passed styleIndex value. This method is called
     * on init() and upon each change to the sytleIndex via its associated
     * cmbStyle JComboBox.
    **/
    private void refreshDisplayOptions(int styleIndex)
    {
        if (styleIndex == Options.Display.STYLE_UML) {
            // UML
            cbxVisSymbols.getModel().setSelected(true);
            cbxAbstractItalic.getModel().setSelected(true);
            cbxStaticUlined.getModel().setSelected(true);
            cbxTypeIsSuffixed.getModel().setSelected(true);

            cbxVisSymbols.getModel().setEnabled(false);
            cbxAbstractItalic.getModel().setEnabled(false);
            cbxStaticUlined.getModel().setEnabled(false);
            cbxTypeIsSuffixed.getModel().setEnabled(false);

        } else if (styleIndex == Options.Display.STYLE_JAVA) {
            // Java
            cbxVisSymbols.getModel().setSelected(false);
            cbxAbstractItalic.getModel().setSelected(false);
            cbxStaticUlined.getModel().setSelected(false);
            cbxTypeIsSuffixed.getModel().setSelected(false);

            cbxVisSymbols.getModel().setEnabled(false);
            cbxAbstractItalic.getModel().setEnabled(false);
            cbxStaticUlined.getModel().setEnabled(false);
            cbxTypeIsSuffixed.getModel().setEnabled(false);

        } else if (styleIndex == Options.Display.STYLE_CUSTOM) {
            // Custom
            cbxVisSymbols.getModel().setEnabled(true);
            cbxAbstractItalic.getModel().setEnabled(true);
            cbxStaticUlined.getModel().setEnabled(true);
            cbxTypeIsSuffixed.getModel().setEnabled(true);

        } else {
            // error, unknown style index
        }

    } // refreshDisplayOptions(int index): void


    public Options getOptions() { return options; }


    /**
     * The method that sets the option object's state to reflect the values
     * specified by the current state of the JBrowseOptionPane.
    **/
    private void setOptions()
    {
        // General Options
        options.setShowStatusBar( cbxStatusBar.getModel().isSelected() );
        options.setAutomaticParse( cbxAutomaticParse.getModel().isSelected() );
        options.setSort( cbxSort.getModel().isSelected() );

        // Filter Options
        filterOpt.setShowAttributes( cbxShowAttributes.getModel().isSelected() );
        filterOpt.setShowPrimitives( cbxShowPrimitives.getModel().isSelected() );
        filterOpt.setShowGeneralizations( cbxShowGeneralizations.getModel().isSelected() );
        filterOpt.setShowThrows( cbxShowThrows.getModel().isSelected() );

        filterOpt.setTopLevelVisIndex( topLevelVisIndex );
        filterOpt.setMemberVisIndex( memberVisIndex );

        // Display Options
        displayOpt.setShowArguments( cbxShowArguments.getModel().isSelected() );
        displayOpt.setShowArgumentNames( cbxShowArgumentNames.getModel().isSelected() );
        displayOpt.setShowNestedName( cbxShowNestedName.getModel().isSelected() );
        displayOpt.setShowIconKeywords( cbxShowIconKeywords.getModel().isSelected() );
        displayOpt.setShowMiscMod( cbxShowMiscMod.getModel().isSelected() );
        displayOpt.setShowLineNum( cbxShowLineNum.getModel().isSelected() );

        displayOpt.setStyleIndex( styleIndex );

        displayOpt.setVisSymbols( cbxVisSymbols.getModel().isSelected() );
        displayOpt.setAbstractItalic( cbxAbstractItalic.getModel().isSelected() );
        displayOpt.setStaticUlined( cbxStaticUlined.getModel().isSelected() );
        displayOpt.setTypeIsSuffixed( cbxTypeIsSuffixed.getModel().isSelected() );

    } // setOptions(): void


    /**
     * Adds a component to the JBrowse option pane.
     * <UL>
     *   <LI>Components are added in a vertical fashion, one per row
     *   <LI>Components fill the horizontal space
     * </UL>
     * Overrides org.gjt.sp.jedit.AbstractOptionPane#addComponent(java.awt.Component)
    **/
    protected void addComponent(Component comp) {
        GridBagConstraints cons = new GridBagConstraints();
        cons.gridy = y++;
        cons.gridheight = 1;
        cons.gridwidth = cons.REMAINDER;
        cons.fill = GridBagConstraints.HORIZONTAL;
        cons.anchor = GridBagConstraints.CENTER;
        cons.weightx = 1.0f;

        gridBag.setConstraints(comp,cons);
        add(comp);
    }


    /**
     * Creates the border of the option panels
    **/
    private Border createOptionBorder(String title) {
        Border border = BorderFactory.createCompoundBorder(
            BorderFactory.createTitledBorder(
                BorderFactory.createBevelBorder(BevelBorder.LOWERED),
                title, TitledBorder.CENTER, TitledBorder.TOP
            ),
            BorderFactory.createEmptyBorder(0, 3, 1, 1)
        );

        return border;
    }


    /**
     * Implements the option pane logic and interactions between components
    **/
    private class UpdateOptionsAction implements ActionListener {
        public void actionPerformed(ActionEvent evt)
        {
            Object actionSource = evt.getSource();

            // Filter Options
            if (actionSource == cbxShowAttributes) {
                if(cbxShowAttributes.getModel().isSelected()) {
                    cbxShowPrimitives.getModel().setEnabled(true);
                } else {
                    cbxShowPrimitives.getModel().setSelected(false);
                    cbxShowPrimitives.getModel().setEnabled(false);
                }
            } else if (actionSource == cmbTopLevelVis) {
                topLevelVisIndex = cmbTopLevelVis.getSelectedIndex();
            } else if (actionSource == cmbMemberVis) {
                memberVisIndex = cmbMemberVis.getSelectedIndex();
            }
            // Display Style Options
            else if (actionSource == cmbStyle) {
                styleIndex = cmbStyle.getSelectedIndex();
                refreshDisplayOptions(styleIndex);
            } else if (actionSource == cbxShowArguments) {
                if(cbxShowArguments.getModel().isSelected()) {
                    cbxShowArgumentNames.getModel().setEnabled(true);
                } else {
                    cbxShowArgumentNames.getModel().setSelected(false);
                    cbxShowArgumentNames.getModel().setEnabled(false);
                }
            }
        }
    }


    /**
     * Action to encapsulate a call to JBrowseOption.setOptions
    **/
    private class SetOptionsAction implements ActionListener {
        public void actionPerformed(ActionEvent evt) {
            JBrowseOptionPane.this.setOptions();
        }
    }


    private class CompoundAction implements ActionListener {
        private ActionListener[] listeners;


        public CompoundAction(ActionListener listener1,
                              ActionListener listener2
        ) {
            this.listeners = new ActionListener[] {
                listener1, listener2
            };
        }


        public CompoundAction(ActionListener[] listeners) {
            this.listeners = listeners;
        }


        public void actionPerformed(ActionEvent evt) {
            for (int i = 0; i < this.listeners.length; i++) {
                if (this.listeners[i] != null) {
                    this.listeners[i].actionPerformed(evt);
                }
            }
        }
    }


    /**
     * Condition interface
    **/
    interface Condition {
        boolean test();
    }


    private class BatchUpdateCondition implements Condition {
        public boolean test() {
            return JBrowseOptionPane.this.batchUpdate;
        }
    }


    private class ConditionalAction implements ActionListener {
        private Condition condition;
        private ActionListener trueAction;
        private ActionListener falseAction;


        public ConditionalAction(Condition condition,
                                 ActionListener trueAction,
                                 ActionListener falseAction
        ) {
            this.condition = condition;
            this.trueAction = trueAction;
            this.falseAction = falseAction;
        }


        public void actionPerformed(ActionEvent evt) {
            if (this.condition.test()) {
                if (this.trueAction != null) {
                    this.trueAction.actionPerformed(evt);
                }
            } else {
                if (this.falseAction != null) {
                    this.falseAction.actionPerformed(evt);
                }
            }
        }
    }


    /**
     * This class is used to for panels that require a gridBag layout for
     * placement into (for example) an OptionPane.
     */
    static class OptionPanel extends JPanel
    {
        /**
         * The layout manager.
         */
        protected GridBagLayout gridBag;

        /**
         * The number of components already added to the layout manager.
         */
        protected int y;

        /**
         * Creates a new option pane.
         * @param name The internal name
         */
        public OptionPanel()
        {
            setLayout(gridBag = new GridBagLayout());
        }


        /**
         * Adds a labeled component to the option pane.
         * @param label The label
         * @param comp The component
         */
        protected void addComponent(String label, Component comp)
        {
            GridBagConstraints cons = new GridBagConstraints();
            cons.gridy = y++;
            cons.gridheight = 1;
            cons.gridwidth = 3;
            cons.fill = GridBagConstraints.BOTH;
            cons.weightx = 1.0f;

            cons.gridx = 0;
            JLabel l = new JLabel(label,SwingConstants.RIGHT);
            gridBag.setConstraints(l,cons);
            add(l);

            cons.gridx = 3;
            cons.gridwidth = 1;
            gridBag.setConstraints(comp,cons);
            add(comp);
        }


        /**
         * Adds a component to the option pane.
         * @param comp The component
         */
        protected void addComponent(Component comp)
        {
            GridBagConstraints cons = new GridBagConstraints();
            cons.gridy = y++;
            cons.gridheight = 1;
            cons.gridwidth = cons.REMAINDER;
            cons.fill = GridBagConstraints.HORIZONTAL;
            cons.anchor = GridBagConstraints.WEST;
            cons.weightx = 1.0f;

            gridBag.setConstraints(comp,cons);
            add(comp);
        }
    }
}


/*
 * ChangeLog:
 * $Log: JBrowseOptionPane.java,v $
 * Revision 1.5  2001/01/18 18:04:33  akaplan
 * Updated for jEdit 3.0, converted to UNIX line endings, some bugs corrected, JBrowse.Activator and JEditActivator removed
 *
 * Revision 1.4  2001/01/18 14:49:20  andre
 * Removed .* imports
 *
 * Revision 1.3  2001/01/08 15:26:33  andre
 * Code cleanings
 *
 * Revision 1.2  2001/01/08 14:32:27  andre
 * Converted line endings to UNIX
 *
 * Revision 1.1.1.1  2000/11/12 14:31:21  andre
 * JBrowse 1.3.1 initial import
 *
 * Revision 1.3  2000/09/02 22:51:46  akaplan
 * JBrowse 1.2: added sort option, automatic reparse feature, important internal changes, see Change_History.html for details
 *
*/
